//class file for A4kDir

#include "cA4kDir.h"
#include <string.h>
#include <iostream.h>

///////////////////////////////////////////////////////
// class A4kDirEntry implemention
///////////////////////////////////////////////////////

bool A4kDirEntry::SetDirEntry(char* RawData)
{   //some checks to avoid program crashes
    if (NULL == RawData) {
        errorcode = A4K_INVALID_POINTER;
        return A4K_NOK;
    }
    if (A4K_DIR_RAW_ENTRY_LENGTH < strlen(RawData)) {
        errorcode = A4K_RAW_DATA_TOO_LONG;
        return  A4K_NOK;
    }
    Occupation=RawData[0];
    for (int i=0;i < A4K_TYPE_LENGTH; i++)
        *(Type+i)=*(RawData+i+1);   //set pointer behind occupation and '\\'
    Type[A4K_TYPE_LENGTH-1]='\0';

    if( !strcmp(Type,"\\SMPL") || !strcmp(Type,"\\SBNK"))
    {   //it is a sample or a bank
        int i=0;
        while(*(Type+i))
        {
            *(Type+i) = *(Type+i+1);    //delete '\\'
            i++;
        }
        Type[A4K_TYPE_LENGTH-1]='\0';   //terminate

        for (int i=0; i< A4K_NAME_LENGTH ; i++) {
            *(Name+i)=*(RawData+i+A4K_TYPE_LENGTH+1);
        }   //fill in type information
        Name[A4K_NAME_LENGTH-1]='\0';

        DirEntry[0]='\0';   //no more usefull information left
    }
    else
    {   //neither a sample nor a bank
        for (int i=1; i< A4K_DIR_RAW_ENTRY_LENGTH ; i++) {
            *(DirEntry+i-1)=*(RawData+i);
        }
        Type[0]='\0';   // delete the wrong type entry
    }
    DirEntry[A4K_DIR_ENTRY_LENGTH-1]='\0';    //terminate string
return A4K_OK;
}

bool A4kDirEntry::GetDirEntry(char* Entry )
{
    if (NULL == Entry) {
        errorcode= A4K_INVALID_POINTER;
        return A4K_NOK;
    }
    if (A4K_DIR_ENTRY_LENGTH > strlen(Entry)){
        errorcode=A4K_STRING_TOO_SHORT;
        return A4K_NOK;
    }
    for(int i=0; i<A4K_DIR_ENTRY_LENGTH ;i++){
        *(Entry+i)=*(DirEntry+i);
    }
return A4K_OK;
}

String A4kDirEntry::GetName(void)
{
        return Name;
}

int A4kDirEntry::GetLastError(void)
{
    return errorcode;
}

String A4kDirEntry::ExtractString(void)
{   //extracts a String out of a Directory Entry
    String s(DirEntry);
    return s;
}

bool A4kDirEntry::IsSample()
{
    return(!strcmp(Type,"SMPL"));
}

bool A4kDirEntry::IsBank()
{
    return(!strcmp(Type,"SBNK"));
}

bool A4kDirEntry::IsEmpty()
{   //true = Directory Entry is empty
    //false = Directory Entry is used
    if(Occupation ==0x00)
        return false;
    else
        return true;
}

A4kDirEntry::A4kDirEntry(void)
{   //constuctor
    Occupation=A4K_DIR_ENTRY_EMPTY;
    for (int i=0;i<A4K_DIR_ENTRY_LENGTH-1;i++){
        DirEntry[i]='\0'; //initialise the class data
    }
}

A4kDirEntry::~A4kDirEntry(void)
{   //destuctor
}

///////////////////////////////////////////////////////
// class A4kDir implemention
///////////////////////////////////////////////////////

A4kDir::A4kDir(void)
{   //constructor
    SymFileName=NULL;
    SampleIndex=0;
    BankIndex=0;
}

A4kDir::~A4kDir(void)
{   //destructor
}

bool A4kDir::CheckFormat(void)
{   //verifies several format specifications
    //and fills public properties
    const String others = "\\OTHERS";

    VolumeName = A4kDirEntries[VOLUME_NAME].ExtractString();
    if(VolumeName==NULL)
    {
        SetError(A4K_NO_VOLUME_NAME_FOUND);
        return A4K_NOK;
    }

    if(others != A4kDirEntries[OTHERS].ExtractString())
    {
        SetError(A4K_NO_OTHERS_FOUND);
        return A4K_NOK;
    }
    return A4K_OK;
}

bool A4kDir::SetDirFile(char* FileName)
{
    try {
        SymFileName = new char[strlen(FileName)+1];
    }
    catch (std::bad_alloc) {
        SetError(A4K_NOT_ENOUGH_MEMORY);
        return A4K_NOK;
    }
    strcpy(SymFileName,FileName);
    return A4K_OK;
}
bool A4kDir::ReadDir(void)
{
        return (LoadSymFile());
}

unsigned int A4kDir::GetNumberOfSamples(void)
{   //delivers amount of samples included in this volume
    NumberOfSamples = 0;
    if(GetFirstSample()!=NULL)
    {
        NumberOfSamples++;
    }
    while(!AllSamplesParsed())
    {
        if(GetNextSample() != NULL)
        {
            NumberOfSamples++;
        }
    }
    return NumberOfSamples;
}

String A4kDir::GetFirstSample(void)
{   //delivers the first sample entry
    SampleIndex=0;
    bAllSamplesParsed = false;
    while(!A4kDirEntries[SampleIndex].IsSample())
    {   //search for first entry which is a sample
        SampleIndex++;
        if(SampleIndex == A4K_MAX_NUM_DIR_ENTRIES)
        {
            bAllSamplesParsed = true;
            return NULL;   //not any sample found, that should never happen
        }
    }
    return (A4kDirEntries[SampleIndex++].GetName());
}
String A4kDir::GetNextSample(void)
{   //delivers the next sample entry
    //allways call GetFirstSample() before
    //call this function until AllSamplesParsed() returns true
    while((!A4kDirEntries[SampleIndex].IsSample())
           && (SampleIndex < A4K_MAX_NUM_DIR_ENTRIES))
    {
        SampleIndex++;
        if(SampleIndex == A4K_MAX_NUM_DIR_ENTRIES)
        {
            bAllSamplesParsed = true;
            return NULL;   //not a sample at this location
        }
    }
    //if we get here, we have found a sample
    return (A4kDirEntries[SampleIndex++].GetName());
}
bool A4kDir::AllSamplesParsed(void)
{   //delivers whether all samples have been fetched
    return bAllSamplesParsed;
}

bool A4kDir::LoadSymFile(void)
{
    int i=0;
    char buffer[A4K_DIR_RAW_ENTRY_LENGTH];
    FILE* SymFile;
    if ((SymFile = fopen(SymFileName,"rb")) == NULL) {
        SetError(A4K_OPEN_FILE_FAILED);
        return A4K_NOK;
    }

    do
    {
        if(!fread(buffer,A4K_DIR_RAW_ENTRY_LENGTH,1,SymFile)) {
        }
        else {
            if(!A4kDirEntries[i].SetDirEntry(buffer)) {
                SetError(GetLastError());
                return A4K_NOK;
            }
        }
        if(i>A4K_MAX_NUM_DIR_ENTRIES) {
            SetError(A4K_ERROR_READING_FILE);
            return A4K_NOK;
        }
        else    i++;

    }
    while(!feof(SymFile));

    //copy information into public properties
    if(A4K_NOK==CheckFormat())  //errorcode is set in CheckFormat();
        return A4K_NOK;
    return A4K_OK;
}

void A4kDir::SetError(int value)
{
    errorcode=value;
}

int A4kDir::GetLastError(void)
{
    return (A4kDirEntry::GetLastError());
}
